"""同步 HTTP 客户端实现"""

import httpx
import threading
from typing import Optional, Dict, Any, Union
from .retry import RetryHandler
from .exceptions import (
    OctenAPIError,
    OctenTimeoutError,
    OctenConnectionError,
    OctenRateLimitError,
    OctenAuthenticationError,
)
from ..utils.constants import (
    DEFAULT_TIMEOUT,
    DEFAULT_MAX_RETRIES,
    DEFAULT_MAX_CONNECTIONS,
    DEFAULT_MAX_KEEPALIVE_CONNECTIONS,
    DEFAULT_KEEPALIVE_EXPIRY,
    HEADER_API_KEY,
    HEADER_CONTENT_TYPE,
    HEADER_USER_AGENT,
    USER_AGENT_TEMPLATE,
)


class HTTPClient:
    """同步 HTTP 客户端

    支持连接池、长连接、HTTP/2 和自动重试。
    使用线程本地存储确保线程安全。

    Attributes:
        base_url: API 基础 URL
        api_key: API 密钥
        timeout: 默认超时时间（秒）
        max_retries: 最大重试次数
        http2: 是否启用 HTTP/2
    """

    def __init__(
        self,
        base_url: str,
        api_key: str,
        timeout: float = DEFAULT_TIMEOUT,
        max_retries: int = DEFAULT_MAX_RETRIES,
        http2: bool = True,
        max_connections: int = DEFAULT_MAX_CONNECTIONS,
        max_keepalive_connections: int = DEFAULT_MAX_KEEPALIVE_CONNECTIONS,
        keepalive_expiry: float = DEFAULT_KEEPALIVE_EXPIRY,
    ):
        self.base_url = base_url
        self.api_key = api_key
        self.timeout = timeout
        self.max_retries = max_retries
        self.http2 = http2
        self.max_connections = max_connections
        self.max_keepalive_connections = max_keepalive_connections
        self.keepalive_expiry = keepalive_expiry

        # 获取版本号
        try:
            from ..version import __version__
            self.user_agent = USER_AGENT_TEMPLATE.format(version=__version__)
        except ImportError:
            self.user_agent = USER_AGENT_TEMPLATE.format(version="unknown")

        # 使用线程本地存储，确保每个线程有独立的客户端实例
        self._local = threading.local()

        # 记录所有创建的客户端，用于清理
        self._clients_lock = threading.Lock()
        self._all_clients = []

        # 初始化重试处理器
        self._retry_handler = RetryHandler(max_retries=max_retries)

    def _get_client(self) -> httpx.Client:
        """获取当前线程的 httpx.Client 实例

        如果当前线程还没有客户端，则创建一个新的。
        这确保了每个线程使用独立的客户端实例，避免线程安全问题。

        Returns:
            当前线程的 httpx.Client 实例
        """
        if not hasattr(self._local, "client") or self._local.client is None:
            # 配置连接池
            limits = httpx.Limits(
                max_connections=self.max_connections,
                max_keepalive_connections=self.max_keepalive_connections,
                keepalive_expiry=self.keepalive_expiry,
            )

            # 构建请求头
            headers = {
                HEADER_API_KEY: self.api_key,
                HEADER_CONTENT_TYPE: "application/json",
                HEADER_USER_AGENT: self.user_agent,
            }

            # 为当前线程创建新的客户端
            self._local.client = httpx.Client(
                base_url=self.base_url,
                timeout=httpx.Timeout(self.timeout),
                http2=self.http2,
                limits=limits,
                headers=headers,
                follow_redirects=True,
            )

            # 记录客户端以便后续清理
            with self._clients_lock:
                self._all_clients.append(self._local.client)

        return self._local.client

    def request(
        self,
        method: str,
        endpoint: str,
        timeout: Optional[float] = None,
        json: Optional[Dict[str, Any]] = None,
        params: Optional[Dict[str, Any]] = None,
        **kwargs,
    ) -> Dict[str, Any]:
        """发送 HTTP 请求

        Args:
            method: HTTP 方法（GET, POST 等）
            endpoint: API 端点路径
            timeout: 请求超时时间（秒），覆盖默认值
            json: JSON 请求体
            params: URL 查询参数
            **kwargs: 其他传递给 httpx 的参数

        Returns:
            响应的 JSON 数据

        Raises:
            OctenAPIError: API 返回错误
            OctenTimeoutError: 请求超时
            OctenConnectionError: 连接错误
        """

        def _make_request() -> Dict[str, Any]:
            try:
                # 使用自定义超时或默认超时
                request_timeout = timeout if timeout is not None else self.timeout

                # 获取当前线程的客户端实例
                client = self._get_client()

                response = client.request(
                    method=method,
                    url=endpoint,
                    json=json,
                    params=params,
                    timeout=request_timeout,
                    **kwargs,
                )

                # 处理响应
                return self._handle_response(response)

            except httpx.TimeoutException as e:
                raise OctenTimeoutError(
                    message=f"Request timed out: {str(e)}",
                    timeout=timeout or self.timeout,
                )
            except httpx.ConnectError as e:
                raise OctenConnectionError(
                    message=f"Connection failed: {str(e)}",
                )
            except httpx.HTTPError as e:
                raise OctenConnectionError(
                    message=f"HTTP error occurred: {str(e)}",
                )

        # 使用重试处理器执行请求
        return self._retry_handler.execute(_make_request)

    def _handle_response(self, response: httpx.Response) -> Dict[str, Any]:
        """处理 HTTP 响应

        Args:
            response: httpx 响应对象

        Returns:
            响应的 JSON 数据

        Raises:
            OctenAPIError: API 返回错误
            OctenRateLimitError: 速率限制错误
            OctenAuthenticationError: 认证错误
        """
        # 提取请求 ID（如果存在）
        request_id = response.headers.get("x-request-id")

        # 尝试解析 JSON 一次（性能优化）
        response_data = self._safe_json(response)

        # 处理成功响应（2xx）
        if response.is_success:
            if response_data is not None:
                return response_data
            else:
                raise OctenAPIError(
                    message="Failed to parse response JSON",
                    status_code=response.status_code,
                    request_id=request_id,
                )

        # 处理错误响应 - 从已解析的 JSON 中提取错误消息
        error_message = self._extract_error_message_from_data(response_data, response)

        # 认证错误（401）
        if response.status_code == 401:
            raise OctenAuthenticationError(
                message=error_message,
                response_data=response_data,
                request_id=request_id,
            )

        # 速率限制错误（429）
        if response.status_code == 429:
            retry_after = response.headers.get("retry-after")
            retry_after_seconds = int(retry_after) if retry_after else None
            raise OctenRateLimitError(
                message=error_message,
                retry_after=retry_after_seconds,
                response_data=response_data,
                request_id=request_id,
            )

        # 其他 API 错误
        raise OctenAPIError(
            message=error_message,
            status_code=response.status_code,
            response_data=response_data,
            request_id=request_id,
        )

    def _extract_error_message_from_data(
        self, data: Optional[Dict[str, Any]], response: httpx.Response
    ) -> str:
        """从已解析的 JSON 数据中提取错误消息

        Args:
            data: 已解析的 JSON 数据
            response: httpx 响应对象

        Returns:
            错误消息字符串
        """
        # 尝试从响应中提取错误消息
        if isinstance(data, dict):
            if "msg" in data:
                return data["msg"]
            if "message" in data:
                return data["message"]
            if "error" in data:
                return str(data["error"])

        # 使用默认错误消息
        return f"HTTP {response.status_code}: {response.reason_phrase}"

    def _safe_json(self, response: httpx.Response) -> Optional[Dict[str, Any]]:
        """安全地解析 JSON 响应

        Args:
            response: httpx 响应对象

        Returns:
            JSON 数据或 None
        """
        try:
            return response.json()
        except Exception:
            return None

    def close(self):
        """关闭所有线程的 HTTP 客户端，释放连接池资源"""
        with self._clients_lock:
            for client in self._all_clients:
                try:
                    client.close()
                except Exception:
                    pass  # 忽略关闭时的错误
            self._all_clients.clear()

        # 清理当前线程的客户端引用
        if hasattr(self._local, "client"):
            self._local.client = None

    def __enter__(self):
        """支持上下文管理器"""
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """退出上下文管理器时自动关闭"""
        self.close()



"""基础用法示例

演示 Octen SDK 的基本功能
"""

from octen import Octen


def main():
    # 创建客户端（使用上下文管理器）
    with Octen(api_key="your-api-key") as client:
        print("=" * 80)
        print("Octen SDK 基础用法示例")
        print("=" * 80)

        # 示例 1: 简单搜索
        print("\n1. 简单搜索")
        print("-" * 80)
        response = client.search.search("Python programming", count=5)

        print(f"查询: {response.query}")
        print(f"搜索类型: {response.search_type}")
        print(f"结果数量: {len(response.results)}\n")

        for i, result in enumerate(response.results, 1):
            print(f"结果 {i}:")
            print(f"  标题: {result['title']}")
            print(f"  URL: {result['url']}")
            if result.get('highlight'):
                print(f"  摘要: {result['highlight'][:100]}...")
            print()

        # 示例 2: 创建文本嵌入
        print("\n2. 创建文本嵌入")
        print("-" * 80)
        embedding_response = client.embedding.create(
            input="Hello, world!",
            model="octen-embedding-4b"
        )

        vector = embedding_response.get_first_embedding()
        if vector:
            print(f"输入文本: Hello, world!")
            print(f"向量维度: {len(vector)}")
            print(f"向量前 5 个值: {vector[:5]}")
            print(f"使用的模型: {embedding_response.model}")

        # 示例 3: 批量嵌入
        print("\n3. 批量文本嵌入")
        print("-" * 80)
        texts = [
            "Python is a programming language",
            "JavaScript is used for web development",
            "Machine learning is a subset of AI"
        ]

        batch_response = client.embedding.create(input=texts)
        vectors = batch_response.get_embeddings()

        print(f"输入文本数量: {len(texts)}")
        print(f"生成向量数量: {len(vectors)}")
        for i, text in enumerate(texts):
            print(f"  {i + 1}. {text} -> 向量维度 {len(vectors[i])}")

        # 示例 4: 使用便捷方法
        print("\n4. 使用便捷方法")
        print("-" * 80)

        # 简单搜索
        simple_response = client.search.simple_search("AI", count=3)
        print(f"简单搜索结果: {len(simple_response.results)} 条")

        # 查询嵌入
        query_vector = client.embedding.embed_query("search query")
        print(f"查询向量维度: {len(query_vector)}")

        # 文档嵌入
        doc_vectors = client.embedding.embed_documents(["doc 1", "doc 2"])
        print(f"文档向量数量: {len(doc_vectors)}")


if __name__ == "__main__":
    main()



"""Octen API 同步客户端"""

from typing import Optional
from .core.http_client import HTTPClient
from .resources.search import SearchResource
from .resources.embedding import EmbeddingResource
from .utils.constants import DEFAULT_BASE_URL, DEFAULT_TIMEOUT, DEFAULT_MAX_RETRIES


class Octen:
    """Octen API 同步客户端

    提供对 Octen API 的便捷访问，支持搜索和嵌入功能。

    Attributes:
        search: Search API 资源
        embedding: Embedding API 资源

    Example:
        基础用法::

            from octen import Octen

            # 使用上下文管理器（推荐）
            with Octen(api_key="your-api-key") as client:
                # 搜索
                response = client.search.search("Python programming", count=5)
                for result in response.results:
                    print(result['title'])

                # 创建嵌入
                embedding = client.embedding.create("Hello, world!")
                vector = embedding.get_first_embedding()

        自定义配置::

            with Octen(
                api_key="your-api-key",
                base_url="https://api.octen.ai",
                timeout=60.0,
                max_retries=5
            ) as client:
                # 使用自定义超时
                response = client.search.search(
                    "complex query",
                    timeout=120.0
                )

        不使用上下文管理器::

            client = Octen(api_key="your-api-key")
            try:
                response = client.search.search("query")
            finally:
                client.close()  # 确保关闭连接
    """

    def __init__(
        self,
        api_key: str,
        base_url: str = DEFAULT_BASE_URL,
        timeout: float = DEFAULT_TIMEOUT,
        max_retries: int = DEFAULT_MAX_RETRIES,
        http2: bool = True,
    ):
        """初始化 Octen 客户端

        Args:
            api_key: Octen API 密钥（必填）
            base_url: API 基础 URL（默认: https://api.octen.ai）
            timeout: 默认请求超时时间，单位秒（默认: 30.0）
            max_retries: 最大重试次数（默认: 3）
            http2: 是否启用 HTTP/2（默认: True）

        Raises:
            ValueError: 如果 api_key 为空
        """
        if not api_key:
            raise ValueError("api_key 不能为空")

        # 创建 HTTP 客户端（核心请求层）
        self._http_client = HTTPClient(
            base_url=base_url,
            api_key=api_key,
            timeout=timeout,
            max_retries=max_retries,
            http2=http2,
        )

        # 初始化资源（业务层）
        self.search = SearchResource(self._http_client)
        self.embedding = EmbeddingResource(self._http_client)

    def close(self):
        """关闭客户端，释放连接池资源

        在不使用上下文管理器时，应该手动调用此方法。
        """
        self._http_client.close()

    def __enter__(self):
        """支持上下文管理器协议"""
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """退出上下文管理器时自动关闭客户端"""
        self.close()
        return False

    def __repr__(self) -> str:
        """返回客户端的字符串表示"""
        return f"Octen(base_url={self._http_client.base_url!r})"



"""Octen API 异步客户端"""

from typing import Optional
from .core.async_http_client import AsyncHTTPClient
from .resources.search import SearchResource
from .resources.embedding import EmbeddingResource
from .utils.constants import DEFAULT_BASE_URL, DEFAULT_TIMEOUT, DEFAULT_MAX_RETRIES


class AsyncSearchResource(SearchResource):
    """异步 Search API 资源

    继承自同步 SearchResource，但使用异步 HTTP 客户端
    """

    async def search(self, *args, **kwargs):
        """异步执行 Web 搜索

        参数与同步版本相同，但需要 await 调用
        """
        # 复用父类的参数验证逻辑
        from octen.models.search import SearchRequest, SearchResponse
        from octen.utils.constants import ENDPOINT_SEARCH

        request = SearchRequest(
            query=kwargs.get("query"),
            count=kwargs.get("count"),
            search_type=kwargs.get("search_type"),
            include_domains=kwargs.get("include_domains"),
            exclude_domains=kwargs.get("exclude_domains"),
            include_text=kwargs.get("include_text"),
            exclude_text=kwargs.get("exclude_text"),
            time_basis=kwargs.get("time_basis"),
            start_time=kwargs.get("start_time"),
            end_time=kwargs.get("end_time"),
            highlight=kwargs.get("highlight"),
            format=kwargs.get("format"),
            safesearch=kwargs.get("safesearch"),
            full_content=kwargs.get("full_content"),
        )

        response_data = await self._client.request(
            method="POST",
            endpoint=ENDPOINT_SEARCH,
            json=request.model_dump(exclude_none=True),
            timeout=kwargs.get("timeout"),
        )

        return SearchResponse(**response_data)

    async def simple_search(self, query: str, count: int = 5, timeout: Optional[float] = None):
        """异步简化搜索接口"""
        return await self.search(query=query, count=count, timeout=timeout)


class AsyncEmbeddingResource(EmbeddingResource):
    """异步 Embedding API 资源

    继承自同步 EmbeddingResource，但使用异步 HTTP 客户端
    """

    async def create(self, *args, **kwargs):
        """异步创建文本嵌入

        参数与同步版本相同，但需要 await 调用
        """
        from octen.models.embedding import EmbeddingRequest, EmbeddingResponse
        from octen.utils.constants import ENDPOINT_EMBEDDING

        request = EmbeddingRequest(
            input=kwargs.get("input"),
            model=kwargs.get("model"),
            dimension=kwargs.get("dimension"),
            input_type=kwargs.get("input_type"),
            truncation=kwargs.get("truncation"),
        )

        response_data = await self._client.request(
            method="POST",
            endpoint=ENDPOINT_EMBEDDING,
            json=request.model_dump(exclude_none=True),
            timeout=kwargs.get("timeout"),
        )

        return EmbeddingResponse(**response_data)

    async def embed_query(self, text: str, model: Optional[str] = None,
                         dimension: Optional[int] = None, timeout: Optional[float] = None):
        """异步嵌入查询文本"""
        response = await self.create(
            input=text,
            model=model,
            dimension=dimension,
            input_type="query",
            timeout=timeout,
        )
        return response.get_first_embedding() or []

    async def embed_documents(self, texts, model: Optional[str] = None,
                             dimension: Optional[int] = None, timeout: Optional[float] = None):
        """异步嵌入文档文本列表"""
        response = await self.create(
            input=texts,
            model=model,
            dimension=dimension,
            input_type="document",
            timeout=timeout,
        )
        return response.get_embeddings()


class AsyncOcten:
    """Octen API 异步客户端

    提供对 Octen API 的异步访问，支持搜索和嵌入功能。

    Attributes:
        search: 异步 Search API 资源
        embedding: 异步 Embedding API 资源

    Example:
        基础用法::

            import asyncio
            from octen import AsyncOcten

            async def main():
                # 使用异步上下文管理器（推荐）
                async with AsyncOcten(api_key="your-api-key") as client:
                    # 异步搜索
                    response = await client.search.search("Python", count=5)
                    for result in response.results:
                        print(result['title'])

                    # 异步嵌入
                    embedding = await client.embedding.create("Hello")
                    vector = embedding.get_first_embedding()

            asyncio.run(main())

        并发请求::

            async def main():
                async with AsyncOcten(api_key="your-api-key") as client:
                    # 同时执行多个请求
                    search_task = client.search.search("AI")
                    embedding_task = client.embedding.create("Hello")

                    results, embeddings = await asyncio.gather(
                        search_task,
                        embedding_task
                    )

            asyncio.run(main())
    """

    def __init__(
        self,
        api_key: str,
        base_url: str = DEFAULT_BASE_URL,
        timeout: float = DEFAULT_TIMEOUT,
        max_retries: int = DEFAULT_MAX_RETRIES,
        http2: bool = True,
    ):
        """初始化异步 Octen 客户端

        Args:
            api_key: Octen API 密钥（必填）
            base_url: API 基础 URL（默认: https://api.octen.ai）
            timeout: 默认请求超时时间，单位秒（默认: 30.0）
            max_retries: 最大重试次数（默认: 3）
            http2: 是否启用 HTTP/2（默认: True）

        Raises:
            ValueError: 如果 api_key 为空
        """
        if not api_key:
            raise ValueError("api_key 不能为空")

        # 创建异步 HTTP 客户端
        self._http_client = AsyncHTTPClient(
            base_url=base_url,
            api_key=api_key,
            timeout=timeout,
            max_retries=max_retries,
            http2=http2,
        )

        # 初始化异步资源
        self.search = AsyncSearchResource(self._http_client)
        self.embedding = AsyncEmbeddingResource(self._http_client)

    async def close(self):
        """关闭客户端，释放连接池资源

        在不使用异步上下文管理器时，应该手动调用此方法。
        """
        await self._http_client.close()

    async def __aenter__(self):
        """支持异步上下文管理器协议"""
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """退出异步上下文管理器时自动关闭客户端"""
        await self.close()
        return False

    def __repr__(self) -> str:
        """返回客户端的字符串表示"""
        return f"AsyncOcten(base_url={self._http_client.base_url!r})"



"""Search API 资源封装"""

from typing import Optional, List
from .base import Resource
from ..models.search import (
    SearchRequest,
    SearchResponse,
    HighlightOptions,
    FullContentOptions,
)
from ..utils.constants import ENDPOINT_SEARCH


class SearchResource(Resource):
    """Search API 资源

    提供 Web 搜索功能
    """

    def search(
        self,
        query: str,
        count: Optional[int] = None,
        search_type: Optional[str] = None,
        include_domains: Optional[List[str]] = None,
        exclude_domains: Optional[List[str]] = None,
        include_text: Optional[List[str]] = None,
        exclude_text: Optional[List[str]] = None,
        time_basis: Optional[str] = None,
        start_time: Optional[str] = None,
        end_time: Optional[str] = None,
        highlight: Optional[HighlightOptions] = None,
        format: Optional[str] = None,
        safesearch: Optional[str] = None,
        full_content: Optional[FullContentOptions] = None,
        timeout: Optional[float] = None,
    ) -> SearchResponse:
        """执行 Web 搜索

        Args:
            query: 搜索查询字符串（必填，最多 500 字符）
            count: 返回结果数量（1-100，默认 5）
            search_type: 搜索类型（auto/keyword/semantic，默认 auto）
            include_domains: 包含的域名列表
            exclude_domains: 排除的域名列表
            include_text: 必须包含的文本列表
            exclude_text: 必须排除的文本列表
            time_basis: 时间基准（auto/published/crawled，默认 auto）
            start_time: 开始时间（ISO 8601 格式）
            end_time: 结束时间（ISO 8601 格式）
            highlight: 高亮选项配置
            format: 内容格式（text/markdown，默认 text）
            safesearch: 安全搜索级别（off/strict，默认 strict）
            full_content: 完整内容选项配置
            timeout: 请求超时时间（秒），覆盖默认值

        Returns:
            SearchResponse: 搜索响应对象

        Raises:
            OctenValidationError: 参数验证失败
            OctenAPIError: API 请求失败
            OctenTimeoutError: 请求超时

        Example:
            >>> from octen import Octen
            >>> client = Octen(api_key="your-api-key")
            >>> response = client.search.search(
            ...     query="Python programming",
            ...     count=10,
            ...     search_type="semantic"
            ... )
            >>> for result in response.results:
            ...     print(result['title'], result['url'])
        """
        # 创建并验证请求模型
        request = SearchRequest(
            query=query,
            count=count,
            search_type=search_type,
            include_domains=include_domains,
            exclude_domains=exclude_domains,
            include_text=include_text,
            exclude_text=exclude_text,
            time_basis=time_basis,
            start_time=start_time,
            end_time=end_time,
            highlight=highlight,
            format=format,
            safesearch=safesearch,
            full_content=full_content,
        )

        # 调用 HTTP 客户端
        response_data = self._client.request(
            method="POST",
            endpoint=ENDPOINT_SEARCH,
            json=request.model_dump(exclude_none=True),
            timeout=timeout,
        )

        # 解析并返回响应
        return SearchResponse(**response_data)

    def simple_search(
        self,
        query: str,
        count: int = 5,
        timeout: Optional[float] = None,
    ) -> SearchResponse:
        """简化的搜索接口

        使用默认参数执行快速搜索

        Args:
            query: 搜索查询字符串
            count: 返回结果数量（默认 5）
            timeout: 请求超时时间（秒）

        Returns:
            SearchResponse: 搜索响应对象

        Example:
            >>> response = client.search.simple_search("Python", count=5)
        """
        return self.search(query=query, count=count, timeout=timeout)



"""资源基类"""

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ..core.http_client import HTTPClient


class Resource:
    """所有资源类的基类

    负责封装 API 端点，提供业务逻辑层接口

    Attributes:
        _client: HTTP 客户端实例
    """

    def __init__(self, client: "HTTPClient"):
        """初始化资源

        Args:
            client: HTTP 客户端实例
        """
        self._client = client



"""Octen SDK 资源模块"""

from .base import Resource
from .search import SearchResource
from .embedding import EmbeddingResource

__all__ = [
    "Resource",
    "SearchResource",
    "EmbeddingResource",
]



"""测试 Embedding API"""

import pytest
from pydantic import ValidationError
from octen.models.embedding import EmbeddingRequest, EmbeddingResponse


def test_embedding_request_single_input():
    """测试单个输入"""
    request = EmbeddingRequest(input="Hello, world!")
    assert request.input == "Hello, world!"
    assert request.truncation is True  # 默认值


def test_embedding_request_batch_input():
    """测试批量输入"""
    texts = ["text 1", "text 2", "text 3"]
    request = EmbeddingRequest(input=texts)
    assert request.input == texts
    assert len(request.input) == 3


def test_embedding_request_with_options():
    """测试带选项的请求"""
    request = EmbeddingRequest(
        input="test",
        model="octen-embedding-4b",
        dimension=768,
        input_type="query",
        truncation=False,
    )
    assert request.model == "octen-embedding-4b"
    assert request.dimension == 768
    assert request.input_type == "query"
    assert request.truncation is False


def test_embedding_request_validation():
    """测试请求验证"""
    # 无效的模型
    with pytest.raises(ValidationError):
        EmbeddingRequest(input="test", model="invalid-model")

    # 无效的输入类型
    with pytest.raises(ValidationError):
        EmbeddingRequest(input="test", input_type="invalid")

    # 无效的维度
    with pytest.raises(ValidationError):
        EmbeddingRequest(input="test", dimension=0)


def test_embedding_response():
    """测试嵌入响应"""
    response_data = {
        "code": 0,
        "msg": "success",
        "data": {
            "model": "octen-embedding-4b",
            "results": [
                {"index": 0, "embedding": [0.1, 0.2, 0.3, 0.4, 0.5]},
                {"index": 1, "embedding": [0.6, 0.7, 0.8, 0.9, 1.0]},
            ],
        },
        "meta": {
            "usage": {"total_tokens": 50},
        },
    }

    response = EmbeddingResponse(**response_data)
    assert response.code == 0
    assert response.msg == "success"
    assert response.model == "octen-embedding-4b"
    assert len(response.results) == 2


def test_embedding_response_get_embeddings():
    """测试获取嵌入向量"""
    response_data = {
        "code": 0,
        "msg": "success",
        "data": {
            "results": [
                {"index": 0, "embedding": [0.1, 0.2, 0.3]},
                {"index": 1, "embedding": [0.4, 0.5, 0.6]},
            ],
        },
        "meta": {},
    }

    response = EmbeddingResponse(**response_data)
    embeddings = response.get_embeddings()

    assert len(embeddings) == 2
    assert embeddings[0] == [0.1, 0.2, 0.3]
    assert embeddings[1] == [0.4, 0.5, 0.6]


def test_embedding_response_get_first_embedding():
    """测试获取第一个嵌入向量"""
    response_data = {
        "code": 0,
        "msg": "success",
        "data": {
            "results": [
                {"index": 0, "embedding": [0.1, 0.2, 0.3]},
            ],
        },
        "meta": {},
    }

    response = EmbeddingResponse(**response_data)
    first_embedding = response.get_first_embedding()

    assert first_embedding == [0.1, 0.2, 0.3]


def test_embedding_response_empty():
    """测试空响应"""
    response_data = {
        "code": 0,
        "msg": "success",
        "data": {"results": []},
        "meta": {},
    }

    response = EmbeddingResponse(**response_data)
    assert response.get_embeddings() == []
    assert response.get_first_embedding() is None



"""异步用法示例

演示 Octen SDK 的异步功能（需要安装 octen[async]）
"""

import asyncio
from octen import AsyncOcten


async def main():
    """异步示例主函数"""
    async with AsyncOcten(api_key="your-api-key") as client:
        print("=" * 80)
        print("Octen SDK 异步用法示例")
        print("=" * 80)

        # 示例 1: 异步搜索
        print("\n1. 异步搜索")
        print("-" * 80)
        response = await client.search.search("Python async", count=5)

        print(f"找到 {len(response.results)} 个结果")
        for result in response.results[:3]:
            print(f"  - {result['title']}")

        # 示例 2: 异步嵌入
        print("\n2. 异步嵌入")
        print("-" * 80)
        embedding = await client.embedding.create("async programming")
        vector = embedding.get_first_embedding()

        if vector:
            print(f"向量维度: {len(vector)}")

        # 示例 3: 并发执行多个请求
        print("\n3. 并发执行")
        print("-" * 80)

        # 同时执行搜索和嵌入
        search_task = client.search.search("AI", count=5)
        embedding_task = client.embedding.create("Hello")

        search_result, embedding_result = await asyncio.gather(
            search_task,
            embedding_task
        )

        print(f"搜索结果: {len(search_result.results)} 条")
        print(f"嵌入维度: {len(embedding_result.get_first_embedding())}")

        # 示例 4: 批量并发搜索
        print("\n4. 批量并发搜索")
        print("-" * 80)

        queries = ["Python", "JavaScript", "Go", "Rust"]
        tasks = [client.search.search(q, count=3) for q in queries]
        results = await asyncio.gather(*tasks)

        for query, result in zip(queries, results):
            print(f"  {query}: {len(result.results)} 个结果")

        # 示例 5: 批量并发嵌入
        print("\n5. 批量并发嵌入")
        print("-" * 80)

        texts = [f"Document {i}" for i in range(10)]
        tasks = [client.embedding.create(text) for text in texts]
        embeddings = await asyncio.gather(*tasks)

        print(f"生成了 {len(embeddings)} 个嵌入向量")


if __name__ == "__main__":
    # 运行异步主函数
    asyncio.run(main())



"""SDK 常量定义"""

# 默认配置
DEFAULT_BASE_URL = "https://api.octen.ai"
DEFAULT_TIMEOUT = 30.0
DEFAULT_MAX_RETRIES = 3

# 连接池配置
DEFAULT_MAX_CONNECTIONS = 100
DEFAULT_MAX_KEEPALIVE_CONNECTIONS = 20
DEFAULT_KEEPALIVE_EXPIRY = 30.0

# 重试配置
RETRY_BASE_DELAY = 1.0
RETRY_MAX_DELAY = 60.0

# API 端点
ENDPOINT_SEARCH = "/search"
ENDPOINT_EMBEDDING = "/embedding"

# 请求头
HEADER_API_KEY = "x-api-key"
HEADER_CONTENT_TYPE = "Content-Type"
HEADER_USER_AGENT = "User-Agent"

# User-Agent
USER_AGENT_TEMPLATE = "octen-python-sdk/{version}"

# 嵌入模型
EMBEDDING_MODELS = {
    "0.6b": "octen-embedding-0.6b",
    "4b": "octen-embedding-4b",
    "8b": "octen-embedding-8b",
}

# 搜索类型
SEARCH_TYPES = {"auto", "keyword", "semantic"}

# 时间基准
TIME_BASIS_OPTIONS = {"auto", "published", "crawled"}

# 格式选项
FORMAT_OPTIONS = {"text", "markdown"}

# 安全搜索
SAFESEARCH_OPTIONS = {"off", "strict"}

# 输入类型
INPUT_TYPES = {"query", "document"}



"""测试 Octen 客户端"""

import pytest
from octen import Octen
from octen.core.exceptions import OctenValidationError


def test_client_initialization():
    """测试客户端初始化"""
    client = Octen(api_key="test-key")
    assert client is not None
    assert client.search is not None
    assert client.embedding is not None
    client.close()


def test_client_context_manager():
    """测试上下文管理器"""
    with Octen(api_key="test-key") as client:
        assert client is not None


def test_client_no_api_key():
    """测试缺少 API key 时抛出异常"""
    with pytest.raises(ValueError, match="api_key 不能为空"):
        Octen(api_key="")


def test_client_custom_config():
    """测试自定义配置"""
    client = Octen(
        api_key="test-key",
        base_url="https://custom.api.com",
        timeout=60.0,
        max_retries=5,
        http2=False,
    )
    assert client._http_client.base_url == "https://custom.api.com"
    assert client._http_client.timeout == 60.0
    assert client._http_client.max_retries == 5
    client.close()


def test_client_repr():
    """测试字符串表示"""
    client = Octen(api_key="test-key", base_url="https://api.octen.ai")
    repr_str = repr(client)
    assert "Octen" in repr_str
    assert "https://api.octen.ai" in repr_str
    client.close()



"""Octen SDK 数据模型"""

from .base import OctenBaseModel
from .search import (
    HighlightOptions,
    FullContentOptions,
    SearchRequest,
    SearchResult,
    SearchResponse,
)
from .embedding import (
    EmbeddingRequest,
    EmbeddingObject,
    EmbeddingResponse,
)

__all__ = [
    "OctenBaseModel",
    "HighlightOptions",
    "FullContentOptions",
    "SearchRequest",
    "SearchResult",
    "SearchResponse",
    "EmbeddingRequest",
    "EmbeddingObject",
    "EmbeddingResponse",
]



"""Octen SDK 异常定义"""

from typing import Optional, Dict, Any


class OctenError(Exception):
    """Octen SDK 基础异常类"""

    def __init__(self, message: str, **kwargs):
        super().__init__(message)
        self.message = message
        for key, value in kwargs.items():
            setattr(self, key, value)


class OctenAPIError(OctenError):
    """API 请求错误

    Attributes:
        message: 错误消息
        status_code: HTTP 状态码
        response_data: 原始响应数据
        request_id: 请求 ID（如果有）
    """

    def __init__(
        self,
        message: str,
        status_code: Optional[int] = None,
        response_data: Optional[Dict[str, Any]] = None,
        request_id: Optional[str] = None,
    ):
        super().__init__(message)
        self.status_code = status_code
        self.response_data = response_data
        self.request_id = request_id

    def __str__(self):
        parts = [self.message]
        if self.status_code:
            parts.append(f"Status: {self.status_code}")
        if self.request_id:
            parts.append(f"Request ID: {self.request_id}")
        return " | ".join(parts)


class OctenTimeoutError(OctenError):
    """请求超时错误"""

    def __init__(self, message: str = "Request timed out", timeout: Optional[float] = None):
        super().__init__(message)
        self.timeout = timeout

    def __str__(self):
        if self.timeout:
            return f"{self.message} (timeout: {self.timeout}s)"
        return self.message


class OctenConnectionError(OctenError):
    """连接错误"""

    def __init__(self, message: str = "Connection error occurred"):
        super().__init__(message)


class OctenValidationError(OctenError):
    """数据验证错误"""

    def __init__(self, message: str, errors: Optional[list] = None):
        super().__init__(message)
        self.errors = errors or []

    def __str__(self):
        if self.errors:
            error_details = "; ".join(str(e) for e in self.errors)
            return f"{self.message}: {error_details}"
        return self.message


class OctenRateLimitError(OctenAPIError):
    """请求速率限制错误"""

    def __init__(
        self,
        message: str = "Rate limit exceeded",
        retry_after: Optional[int] = None,
        **kwargs,
    ):
        super().__init__(message, status_code=429, **kwargs)
        self.retry_after = retry_after


class OctenAuthenticationError(OctenAPIError):
    """认证错误"""

    def __init__(self, message: str = "Authentication failed", **kwargs):
        super().__init__(message, status_code=401, **kwargs)



"""Embedding API 数据模型"""

from typing import Union, List, Optional, Dict, Any
from pydantic import Field, field_validator
from .base import OctenBaseModel
from ..utils.constants import EMBEDDING_MODELS, INPUT_TYPES


class EmbeddingRequest(OctenBaseModel):
    """Embedding API 请求模型

    Attributes:
        input: 输入文本（字符串或字符串列表）
        model: 模型名称（octen-embedding-0.6b/4b/8b）
        dimension: 向量维度
        input_type: 输入类型（query/document）
        truncation: 是否截断超长输入
    """

    input: Union[str, List[str]] = Field(
        ...,
        description="输入文本",
    )
    model: Optional[str] = Field(
        None,
        description="模型: octen-embedding-0.6b/4b/8b",
    )
    dimension: Optional[int] = Field(
        None,
        ge=1,
        description="向量维度",
    )
    input_type: Optional[str] = Field(
        None,
        description="输入类型: query/document",
    )
    truncation: Optional[bool] = Field(
        True,
        description="是否截断超长输入",
    )

    @field_validator("model")
    @classmethod
    def validate_model(cls, v: Optional[str]) -> Optional[str]:
        """验证模型名称"""
        if v is not None:
            valid_models = set(EMBEDDING_MODELS.values())
            if v not in valid_models:
                raise ValueError(f"model 必须是 {valid_models} 之一")
        return v

    @field_validator("input_type")
    @classmethod
    def validate_input_type(cls, v: Optional[str]) -> Optional[str]:
        """验证输入类型"""
        if v is not None and v not in INPUT_TYPES:
            raise ValueError(f"input_type 必须是 {INPUT_TYPES} 之一")
        return v


class EmbeddingObject(OctenBaseModel):
    """单个嵌入对象

    Attributes:
        index: 在批次中的索引
        embedding: 向量表示
    """

    index: int = Field(..., description="索引")
    embedding: List[float] = Field(..., description="向量")

    @property
    def dimension(self) -> int:
        """获取向量维度"""
        return len(self.embedding)


class EmbeddingResponse(OctenBaseModel):
    """Embedding API 响应模型

    Attributes:
        code: 响应码（0 表示成功）
        msg: 响应消息
        data: 响应数据
        meta: 元数据（包含 usage、warning 等）
    """

    code: int = Field(..., description="响应码")
    msg: str = Field(..., description="响应消息")
    data: Dict[str, Any] = Field(..., description="响应数据")
    meta: Dict[str, Any] = Field(..., description="元数据")

    @property
    def results(self) -> List[Dict[str, Any]]:
        """获取嵌入结果列表"""
        return self.data.get("results", [])

    @property
    def model(self) -> Optional[str]:
        """获取实际使用的模型"""
        return self.data.get("model")

    @property
    def usage(self) -> Optional[Dict[str, Any]]:
        """获取使用量信息"""
        return self.meta.get("usage")

    @property
    def warning(self) -> Optional[str]:
        """获取警告信息"""
        return self.meta.get("warning")

    def get_embeddings(self) -> List[List[float]]:
        """获取所有嵌入向量

        Returns:
            嵌入向量列表
        """
        return [result["embedding"] for result in self.results if "embedding" in result]

    def get_first_embedding(self) -> Optional[List[float]]:
        """获取第一个嵌入向量（单输入时使用）

        Returns:
            第一个嵌入向量，如果没有则返回 None
        """
        embeddings = self.get_embeddings()
        return embeddings[0] if embeddings else None



"""高级用法示例

演示 Octen SDK 的高级功能，包括过滤、高亮、完整内容等
"""

from octen import Octen, HighlightOptions, FullContentOptions


def main():
    with Octen(api_key="your-api-key") as client:
        print("=" * 80)
        print("Octen SDK 高级用法示例")
        print("=" * 80)

        # 示例 1: 高级搜索 - 域名过滤
        print("\n1. 域名过滤搜索")
        print("-" * 80)
        response = client.search.search(
            query="machine learning tutorials",
            count=10,
            include_domains=["github.com", "arxiv.org", "medium.com"],
            exclude_domains=["pinterest.com"]
        )

        print(f"找到 {len(response.results)} 个结果")
        for result in response.results[:3]:
            print(f"  - {result['title']}")
            print(f"    {result['url']}")

        # 示例 2: 时间范围搜索
        print("\n2. 时间范围搜索")
        print("-" * 80)
        response = client.search.search(
            query="Python 3.12 新特性",
            count=5,
            time_basis="published",
            start_time="2024-01-01T00:00:00Z",
            end_time="2024-12-31T23:59:59Z"
        )

        print(f"找到 {len(response.results)} 个结果")
        for result in response.results:
            pub_time = result.get('time_published', 'Unknown')
            print(f"  - {result['title']}")
            print(f"    发布时间: {pub_time}")

        # 示例 3: 自定义高亮
        print("\n3. 自定义高亮选项")
        print("-" * 80)
        response = client.search.search(
            query="deep learning",
            count=3,
            highlight=HighlightOptions(
                enable=True,
                max_tokens=500  # 更长的摘要
            )
        )

        for i, result in enumerate(response.results, 1):
            print(f"\n结果 {i}:")
            print(f"标题: {result['title']}")
            if result.get('highlight'):
                print(f"摘要 ({len(result['highlight'])} 字符):")
                print(f"{result['highlight'][:200]}...")

        # 示例 4: 获取完整内容
        print("\n4. 获取完整内容")
        print("-" * 80)
        response = client.search.search(
            query="API design best practices",
            count=2,
            full_content=FullContentOptions(
                enable=True,
                max_tokens=2000
            )
        )

        for i, result in enumerate(response.results, 1):
            print(f"\n结果 {i}: {result['title']}")
            if result.get('full_content'):
                content_length = len(result['full_content'])
                print(f"完整内容长度: {content_length} 字符")
                print(f"内容预览: {result['full_content'][:150]}...")

        # 示例 5: 语义搜索
        print("\n5. 语义搜索")
        print("-" * 80)
        response = client.search.search(
            query="如何提高代码质量",
            count=5,
            search_type="semantic",  # 使用语义搜索
            format="markdown"  # 返回 Markdown 格式
        )

        print(f"搜索类型: {response.search_type}")
        print(f"结果数量: {len(response.results)}")
        for result in response.results[:3]:
            print(f"  - {result['title']}")

        # 示例 6: 文本过滤
        print("\n6. 文本内容过滤")
        print("-" * 80)
        response = client.search.search(
            query="Python web framework",
            count=5,
            include_text=["Django", "Flask"],  # 必须包含这些词
            exclude_text=["deprecated", "outdated"]  # 排除这些词
        )

        print(f"找到 {len(response.results)} 个结果")
        for result in response.results:
            print(f"  - {result['title']}")

        # 示例 7: 自定义超时
        print("\n7. 自定义超时设置")
        print("-" * 80)
        try:
            # 这个请求使用 120 秒超时
            response = client.search.search(
                query="complex query",
                count=20,
                timeout=120.0
            )
            print(f"请求成功，找到 {len(response.results)} 个结果")
        except Exception as e:
            print(f"请求失败: {e}")

        # 示例 8: 查看使用量信息
        print("\n8. 查看使用量信息")
        print("-" * 80)
        response = client.search.search("test query", count=3)

        if response.usage:
            print(f"Token 使用量:")
            for key, value in response.usage.items():
                print(f"  {key}: {value}")

        if response.latency:
            print(f"\n延迟信息:")
            for key, value in response.latency.items():
                print(f"  {key}: {value}ms")


if __name__ == "__main__":
    main()



"""重试逻辑处理"""

import time
from typing import Callable, TypeVar, Set
from .exceptions import OctenAPIError, OctenTimeoutError, OctenConnectionError

T = TypeVar("T")


class RetryHandler:
    """重试处理器，支持指数退避策略

    Attributes:
        max_retries: 最大重试次数
        base_delay: 基础延迟时间（秒）
        max_delay: 最大延迟时间（秒）
    """

    # 可重试的 HTTP 状态码
    RETRYABLE_STATUS_CODES: Set[int] = {
        408,  # Request Timeout
        429,  # Too Many Requests
        500,  # Internal Server Error
        502,  # Bad Gateway
        503,  # Service Unavailable
        504,  # Gateway Timeout
    }

    def __init__(
        self,
        max_retries: int = 3,
        base_delay: float = 1.0,
        max_delay: float = 60.0,
    ):
        self.max_retries = max_retries
        self.base_delay = base_delay
        self.max_delay = max_delay

    def execute(self, func: Callable[[], T]) -> T:
        """执行函数，失败时自动重试

        Args:
            func: 要执行的函数

        Returns:
            函数的返回值

        Raises:
            最后一次尝试的异常
        """
        last_exception = None

        for attempt in range(self.max_retries + 1):
            try:
                return func()
            except (OctenAPIError, OctenTimeoutError, OctenConnectionError) as e:
                last_exception = e

                # 检查是否应该重试
                if not self._should_retry(e, attempt):
                    raise

                # 如果不是最后一次尝试，等待后重试
                if attempt < self.max_retries:
                    delay = self._calculate_delay(attempt, e)
                    time.sleep(delay)

        # 所有重试都失败，抛出最后一个异常
        if last_exception:
            raise last_exception

    def _should_retry(self, error: Exception, attempt: int) -> bool:
        """判断是否应该重试

        Args:
            error: 发生的错误
            attempt: 当前尝试次数（从 0 开始）

        Returns:
            是否应该重试
        """
        # 已达最大重试次数
        if attempt >= self.max_retries:
            return False

        # 超时错误可重试
        if isinstance(error, OctenTimeoutError):
            return True

        # 连接错误可重试
        if isinstance(error, OctenConnectionError):
            return True

        # API 错误根据状态码判断
        if isinstance(error, OctenAPIError):
            if error.status_code in self.RETRYABLE_STATUS_CODES:
                return True

        return False

    def _calculate_delay(self, attempt: int, error: Exception) -> float:
        """计算延迟时间（指数退避）

        Args:
            attempt: 当前尝试次数（从 0 开始）
            error: 发生的错误

        Returns:
            延迟时间（秒）
        """
        # 如果是速率限制错误且有 retry_after 头，使用它
        if isinstance(error, OctenAPIError) and error.status_code == 429:
            if hasattr(error, "retry_after") and error.retry_after:
                return min(error.retry_after, self.max_delay)

        # 指数退避: base_delay * 2^attempt
        delay = self.base_delay * (2**attempt)

        # 限制最大延迟
        return min(delay, self.max_delay)


class AsyncRetryHandler(RetryHandler):
    """异步重试处理器"""

    async def execute_async(self, func: Callable[[], T]) -> T:
        """异步执行函数，失败时自动重试

        Args:
            func: 要执行的异步函数

        Returns:
            函数的返回值

        Raises:
            最后一次尝试的异常
        """
        import asyncio

        last_exception = None

        for attempt in range(self.max_retries + 1):
            try:
                return await func()
            except (OctenAPIError, OctenTimeoutError, OctenConnectionError) as e:
                last_exception = e

                # 检查是否应该重试
                if not self._should_retry(e, attempt):
                    raise

                # 如果不是最后一次尝试，等待后重试
                if attempt < self.max_retries:
                    delay = self._calculate_delay(attempt, e)
                    await asyncio.sleep(delay)

        # 所有重试都失败，抛出最后一个异常
        if last_exception:
            raise last_exception



"""Embedding API 资源封装"""

from typing import Union, List, Optional
from .base import Resource
from ..models.embedding import EmbeddingRequest, EmbeddingResponse
from ..utils.constants import ENDPOINT_EMBEDDING


class EmbeddingResource(Resource):
    """Embedding API 资源

    提供文本嵌入功能
    """

    def create(
        self,
        input: Union[str, List[str]],
        model: Optional[str] = None,
        dimension: Optional[int] = None,
        input_type: Optional[str] = None,
        truncation: Optional[bool] = None,
        timeout: Optional[float] = None,
    ) -> EmbeddingResponse:
        """创建文本嵌入向量

        Args:
            input: 输入文本（字符串或字符串列表）
            model: 模型名称（octen-embedding-0.6b/4b/8b）
            dimension: 向量维度
            input_type: 输入类型（query/document）
            truncation: 是否截断超长输入（默认 True）
            timeout: 请求超时时间（秒），覆盖默认值

        Returns:
            EmbeddingResponse: 嵌入响应对象

        Raises:
            OctenValidationError: 参数验证失败
            OctenAPIError: API 请求失败
            OctenTimeoutError: 请求超时

        Example:
            >>> from octen import Octen
            >>> client = Octen(api_key="your-api-key")
            >>>
            >>> # 单个文本
            >>> response = client.embedding.create(
            ...     input="Hello, world!",
            ...     model="octen-embedding-4b"
            ... )
            >>> vector = response.get_first_embedding()
            >>> print(f"维度: {len(vector)}")
            >>>
            >>> # 批量文本
            >>> response = client.embedding.create(
            ...     input=["text 1", "text 2", "text 3"],
            ...     model="octen-embedding-8b"
            ... )
            >>> vectors = response.get_embeddings()
            >>> print(f"生成了 {len(vectors)} 个向量")
        """
        # 创建并验证请求模型
        request = EmbeddingRequest(
            input=input,
            model=model,
            dimension=dimension,
            input_type=input_type,
            truncation=truncation,
        )

        # 调用 HTTP 客户端
        response_data = self._client.request(
            method="POST",
            endpoint=ENDPOINT_EMBEDDING,
            json=request.model_dump(exclude_none=True),
            timeout=timeout,
        )

        # 解析并返回响应
        return EmbeddingResponse(**response_data)

    def embed_query(
        self,
        text: str,
        model: Optional[str] = None,
        dimension: Optional[int] = None,
        timeout: Optional[float] = None,
    ) -> List[float]:
        """嵌入查询文本（便捷方法）

        Args:
            text: 查询文本
            model: 模型名称
            dimension: 向量维度
            timeout: 请求超时时间（秒）

        Returns:
            向量列表

        Example:
            >>> vector = client.embedding.embed_query("search query")
        """
        response = self.create(
            input=[text],  # 修正：包装为列表
            model=model,
            dimension=dimension,
            input_type="query",
            timeout=timeout,
        )
        return response.get_first_embedding() or []

    def embed_documents(
        self,
        texts: List[str],
        model: Optional[str] = None,
        dimension: Optional[int] = None,
        timeout: Optional[float] = None,
    ) -> List[List[float]]:
        """嵌入文档文本列表（便捷方法）

        Args:
            texts: 文档文本列表
            model: 模型名称
            dimension: 向量维度
            timeout: 请求超时时间（秒）

        Returns:
            向量列表的列表

        Example:
            >>> vectors = client.embedding.embed_documents(
            ...     ["doc 1", "doc 2", "doc 3"]
            ... )
        """
        response = self.create(
            input=texts,
            model=model,
            dimension=dimension,
            input_type="document",
            timeout=timeout,
        )
        return response.get_embeddings()



"""Octen SDK 核心模块"""

from .exceptions import (
    OctenError,
    OctenAPIError,
    OctenTimeoutError,
    OctenConnectionError,
    OctenValidationError,
    OctenRateLimitError,
    OctenAuthenticationError,
)
from .http_client import HTTPClient
from .retry import RetryHandler, AsyncRetryHandler

__all__ = [
    "OctenError",
    "OctenAPIError",
    "OctenTimeoutError",
    "OctenConnectionError",
    "OctenValidationError",
    "OctenRateLimitError",
    "OctenAuthenticationError",
    "HTTPClient",
    "RetryHandler",
    "AsyncRetryHandler",
]



"""测试 Search API"""

import pytest
from pydantic import ValidationError
from octen.models.search import (
    SearchRequest,
    SearchResponse,
    HighlightOptions,
    FullContentOptions,
)


def test_search_request_basic():
    """测试基础搜索请求"""
    request = SearchRequest(query="test query")
    assert request.query == "test query"
    assert request.count == 5  # 默认值
    assert request.search_type == "auto"  # 默认值


def test_search_request_full():
    """测试完整搜索请求"""
    request = SearchRequest(
        query="Python",
        count=10,
        search_type="semantic",
        include_domains=["github.com"],
        exclude_domains=["pinterest.com"],
        highlight=HighlightOptions(enable=True, max_tokens=300),
        full_content=FullContentOptions(enable=True, max_tokens=1000),
    )
    assert request.query == "Python"
    assert request.count == 10
    assert request.search_type == "semantic"
    assert request.include_domains == ["github.com"]


def test_search_request_validation():
    """测试请求验证"""
    # 查询字符串过长
    with pytest.raises(ValidationError):
        SearchRequest(query="x" * 501)

    # 结果数量超出范围
    with pytest.raises(ValidationError):
        SearchRequest(query="test", count=0)

    with pytest.raises(ValidationError):
        SearchRequest(query="test", count=101)

    # 无效的搜索类型
    with pytest.raises(ValidationError):
        SearchRequest(query="test", search_type="invalid")


def test_highlight_options():
    """测试高亮选项"""
    options = HighlightOptions(enable=True, max_tokens=500)
    assert options.enable is True
    assert options.max_tokens == 500

    # 测试默认值
    options_default = HighlightOptions()
    assert options_default.enable is True
    assert options_default.max_tokens == 300


def test_full_content_options():
    """测试完整内容选项"""
    options = FullContentOptions(enable=True, max_tokens=2000)
    assert options.enable is True
    assert options.max_tokens == 2000

    # 测试默认值
    options_default = FullContentOptions()
    assert options_default.enable is False
    assert options_default.max_tokens == 1000


def test_search_response():
    """测试搜索响应"""
    response_data = {
        "code": 0,
        "msg": "success",
        "data": {
            "query": "test",
            "search_type": "auto",
            "results": [
                {
                    "title": "Test Result",
                    "url": "https://example.com",
                    "highlight": "Test highlight",
                }
            ],
        },
        "meta": {
            "usage": {"total_tokens": 100},
            "latency": {"total": 500},
        },
    }

    response = SearchResponse(**response_data)
    assert response.code == 0
    assert response.msg == "success"
    assert len(response.results) == 1
    assert response.results[0]["title"] == "Test Result"
    assert response.query == "test"
    assert response.search_type == "auto"
    assert response.usage["total_tokens"] == 100
    assert response.latency["total"] == 500



"""Search API 数据模型"""

from typing import Optional, List, Dict, Any
from pydantic import Field, field_validator
from .base import OctenBaseModel
from ..utils.constants import (
    SEARCH_TYPES,
    TIME_BASIS_OPTIONS,
    FORMAT_OPTIONS,
    SAFESEARCH_OPTIONS,
)


class HighlightOptions(OctenBaseModel):
    """高亮选项配置

    Attributes:
        enable: 是否返回高亮内容
        max_tokens: 高亮内容的最大 token 数
    """

    enable: bool = Field(True, description="是否返回高亮")
    max_tokens: int = Field(
        300,
        ge=100,
        le=2048,
        description="高亮最大 tokens",
    )


class FullContentOptions(OctenBaseModel):
    """完整内容选项配置

    Attributes:
        enable: 是否返回完整内容
        max_tokens: 完整内容的最大 token 数
    """

    enable: bool = Field(False, description="是否返回完整内容")
    max_tokens: int = Field(
        1000,
        ge=1,
        le=100000,
        description="内容最大 tokens",
    )


class SearchRequest(OctenBaseModel):
    """Search API 请求模型

    Attributes:
        query: 搜索查询字符串
        search_type: 搜索类型（auto/keyword/semantic）
        count: 返回结果数量
        include_domains: 包含的域名列表
        exclude_domains: 排除的域名列表
        include_text: 必须包含的文本列表
        exclude_text: 必须排除的文本列表
        time_basis: 时间基准（auto/published/crawled）
        start_time: 开始时间（ISO 8601 格式）
        end_time: 结束时间（ISO 8601 格式）
        highlight: 高亮选项
        format: 内容格式（text/markdown）
        safesearch: 安全搜索级别（off/strict）
        full_content: 完整内容选项
    """

    query: str = Field(
        ...,
        max_length=500,
        description="搜索查询",
    )
    search_type: Optional[str] = Field(
        "auto",
        description="搜索类型: auto/keyword/semantic",
    )
    count: Optional[int] = Field(
        5,
        ge=1,
        le=100,
        description="结果数量",
    )
    include_domains: Optional[List[str]] = Field(
        None,
        description="包含域名",
    )
    exclude_domains: Optional[List[str]] = Field(
        None,
        description="排除域名",
    )
    include_text: Optional[List[str]] = Field(
        None,
        description="必须包含的文本",
    )
    exclude_text: Optional[List[str]] = Field(
        None,
        description="必须排除的文本",
    )
    time_basis: Optional[str] = Field(
        "auto",
        description="时间基准: auto/published/crawled",
    )
    start_time: Optional[str] = Field(
        None,
        description="开始时间 ISO 8601",
    )
    end_time: Optional[str] = Field(
        None,
        description="结束时间 ISO 8601",
    )
    highlight: Optional[HighlightOptions] = Field(
        None,
        description="高亮选项",
    )
    format: Optional[str] = Field(
        "text",
        description="格式: text/markdown",
    )
    safesearch: Optional[str] = Field(
        "strict",
        description="安全搜索: off/strict",
    )
    full_content: Optional[FullContentOptions] = Field(
        None,
        description="完整内容选项",
    )

    @field_validator("search_type")
    @classmethod
    def validate_search_type(cls, v: Optional[str]) -> Optional[str]:
        """验证搜索类型"""
        if v is not None and v not in SEARCH_TYPES:
            raise ValueError(f"search_type 必须是 {SEARCH_TYPES} 之一")
        return v

    @field_validator("time_basis")
    @classmethod
    def validate_time_basis(cls, v: Optional[str]) -> Optional[str]:
        """验证时间基准"""
        if v is not None and v not in TIME_BASIS_OPTIONS:
            raise ValueError(f"time_basis 必须是 {TIME_BASIS_OPTIONS} 之一")
        return v

    @field_validator("format")
    @classmethod
    def validate_format(cls, v: Optional[str]) -> Optional[str]:
        """验证格式"""
        if v is not None and v not in FORMAT_OPTIONS:
            raise ValueError(f"format 必须是 {FORMAT_OPTIONS} 之一")
        return v

    @field_validator("safesearch")
    @classmethod
    def validate_safesearch(cls, v: Optional[str]) -> Optional[str]:
        """验证安全搜索"""
        if v is not None and v not in SAFESEARCH_OPTIONS:
            raise ValueError(f"safesearch 必须是 {SAFESEARCH_OPTIONS} 之一")
        return v


class SearchResult(OctenBaseModel):
    """搜索结果项

    Attributes:
        title: 页面标题
        url: 页面 URL
        highlight: 高亮摘要
        full_content: 完整内容
        authors: 作者信息
        time_published: 发布时间
        time_last_crawled: 最后爬取时间
    """

    title: str = Field(..., description="标题")
    url: str = Field(..., description="URL")
    highlight: Optional[str] = Field(None, description="高亮摘要")
    full_content: Optional[str] = Field(None, description="完整内容")
    authors: Optional[str] = Field(None, description="作者")
    time_published: Optional[str] = Field(None, description="发布时间")
    time_last_crawled: Optional[str] = Field(None, description="最后爬取时间")


class SearchResponse(OctenBaseModel):
    """Search API 响应模型

    Attributes:
        code: 响应码（0 表示成功）
        msg: 响应消息
        data: 响应数据
        meta: 元数据（包含 usage、latency 等）
    """

    code: int = Field(..., description="响应码")
    msg: str = Field(..., description="响应消息")
    data: Dict[str, Any] = Field(..., description="响应数据")
    meta: Dict[str, Any] = Field(..., description="元数据")

    @property
    def results(self) -> List[Dict[str, Any]]:
        """获取搜索结果列表"""
        return self.data.get("results", [])

    @property
    def query(self) -> Optional[str]:
        """获取查询字符串"""
        return self.data.get("query")

    @property
    def search_type(self) -> Optional[str]:
        """获取实际使用的搜索类型"""
        return self.data.get("search_type")

    @property
    def usage(self) -> Optional[Dict[str, Any]]:
        """获取使用量信息"""
        return self.meta.get("usage")

    @property
    def latency(self) -> Optional[Dict[str, Any]]:
        """获取延迟信息"""
        return self.meta.get("latency")

    @property
    def warning(self) -> Optional[str]:
        """获取警告信息"""
        return self.meta.get("warning")



"""Octen Python SDK

一个用于访问 Octen API 的 Python SDK，提供搜索和嵌入功能。

基础用法::

    from octen import Octen

    with Octen(api_key="your-api-key") as client:
        # 搜索
        response = client.search.search("Python", count=5)
        for result in response.results:
            print(result['title'])

        # 嵌入
        embedding = client.embedding.create("Hello")
        vector = embedding.get_first_embedding()
"""

from .version import __version__, __author__, __license__
from .client import Octen

# 核心异常
from .core.exceptions import (
    OctenError,
    OctenAPIError,
    OctenTimeoutError,
    OctenConnectionError,
    OctenValidationError,
    OctenRateLimitError,
    OctenAuthenticationError,
)

# 数据模型
from .models.search import (
    HighlightOptions,
    FullContentOptions,
    SearchRequest,
    SearchResponse,
)
from .models.embedding import (
    EmbeddingRequest,
    EmbeddingResponse,
)

# 尝试导入异步客户端（如果已实现）
try:
    from .async_client import AsyncOcten

    __all__ = [
        # 版本信息
        "__version__",
        "__author__",
        "__license__",
        # 客户端
        "Octen",
        "AsyncOcten",
        # 异常
        "OctenError",
        "OctenAPIError",
        "OctenTimeoutError",
        "OctenConnectionError",
        "OctenValidationError",
        "OctenRateLimitError",
        "OctenAuthenticationError",
        # 数据模型
        "HighlightOptions",
        "FullContentOptions",
        "SearchRequest",
        "SearchResponse",
        "EmbeddingRequest",
        "EmbeddingResponse",
    ]
except ImportError:
    __all__ = [
        # 版本信息
        "__version__",
        "__author__",
        "__license__",
        # 客户端
        "Octen",
        # 异常
        "OctenError",
        "OctenAPIError",
        "OctenTimeoutError",
        "OctenConnectionError",
        "OctenValidationError",
        "OctenRateLimitError",
        "OctenAuthenticationError",
        # 数据模型
        "HighlightOptions",
        "FullContentOptions",
        "SearchRequest",
        "SearchResponse",
        "EmbeddingRequest",
        "EmbeddingResponse",
    ]



"""Pydantic 基础模型"""

from typing import Any, Dict
from pydantic import BaseModel, ConfigDict


class OctenBaseModel(BaseModel):
    """所有 Octen 数据模型的基类

    配置:
        - 允许额外字段（API 可能返回未知字段）
        - 使用别名生成器支持驼峰命名
        - 冻结模型以确保不可变性
    """

    model_config = ConfigDict(
        extra="allow",  # 允许额外字段
        populate_by_name=True,  # 支持字段别名
        validate_assignment=True,  # 赋值时验证
    )

    def to_dict(self, exclude_none: bool = True) -> Dict[str, Any]:
        """转换为字典

        Args:
            exclude_none: 是否排除 None 值

        Returns:
            字典表示
        """
        return self.model_dump(exclude_none=exclude_none, by_alias=True)
